@@ -3,14 +3,17 @@
 from ..utils.io import *
 from ..utils.print import *
 from .utils import *
+from .shared_import import *
 from .book_optimizer import *
 from ..constants import *
+from copy import deepcopy
 
 def main(config: dict):
 
 	# Prework
 	os.makedirs(f"{config['manual_path']}/font/page", exist_ok=True)
 	os.makedirs(f"{config['manual_path']}/font/wiki_icons", exist_ok = True)
+	os.makedirs(f"{config['manual_path']}/font/high_res", exist_ok = True)
 	generate_all_iso_renders(config)
 
 	# Constants
@@ -22,13 +25,18 @@ def main(config: dict):
 
 	# Copy assets in the resource pack
 	if not config['debug_mode']:
-		super_copy(f"{MANUAL_ASSETS_PATH}/none_release.png", f"{config['build_resource_pack']}/assets/{config['namespace']}/textures/font/none.png")
+		super_copy(f"{TEMPLATES_PATH}/none_release.png", f"{config['build_resource_pack']}/assets/{config['namespace']}/textures/font/none.png")
+		super_copy(f"{TEMPLATES_PATH}/invisible_item_release.png", f"{config['build_resource_pack']}/assets/{config['namespace']}/textures/font/invisible_item.png")
 	else:
-		super_copy(f"{MANUAL_ASSETS_PATH}/none.png", f"{config['build_resource_pack']}/assets/{config['namespace']}/textures/font/none.png")
-	super_copy(f"{MANUAL_ASSETS_PATH}/wiki_information.png", f"{config['build_resource_pack']}/assets/{config['namespace']}/textures/font/wiki_information.png")
-	super_copy(f"{MANUAL_ASSETS_PATH}/wiki_result_of_craft.png", f"{config['build_resource_pack']}/assets/{config['namespace']}/textures/font/wiki_result_of_craft.png")
-	super_copy(f"{MANUAL_ASSETS_PATH}/wiki_ingredient_of_craft.png", f"{config['build_resource_pack']}/assets/{config['namespace']}/textures/font/wiki_ingredient_of_craft.png")
-
+		super_copy(f"{TEMPLATES_PATH}/none.png", f"{config['build_resource_pack']}/assets/{config['namespace']}/textures/font/none.png")
+		super_copy(f"{TEMPLATES_PATH}/invisible_item.png", f"{config['build_resource_pack']}/assets/{config['namespace']}/textures/font/invisible_item.png")
+	super_copy(f"{TEMPLATES_PATH}/wiki_information.png", f"{config['build_resource_pack']}/assets/{config['namespace']}/textures/font/wiki_information.png")
+	super_copy(f"{TEMPLATES_PATH}/wiki_result_of_craft.png", f"{config['build_resource_pack']}/assets/{config['namespace']}/textures/font/wiki_result_of_craft.png")
+	super_copy(f"{TEMPLATES_PATH}/wiki_ingredient_of_craft.png", f"{config['build_resource_pack']}/assets/{config['namespace']}/textures/font/wiki_ingredient_of_craft.png")
+	if config['manual_high_resolution']:
+		super_copy(f"{TEMPLATES_PATH}/furnace.png", f"{config['build_resource_pack']}/assets/{config['namespace']}/textures/font/furnace.png")
+		super_copy(f"{TEMPLATES_PATH}/shaped_2x2.png", f"{config['build_resource_pack']}/assets/{config['namespace']}/textures/font/shaped_2x2.png")
+		super_copy(f"{TEMPLATES_PATH}/shaped_3x3.png", f"{config['build_resource_pack']}/assets/{config['namespace']}/textures/font/shaped_3x3.png")
 
 	# If the manual cache is enabled and we have a cache file, load it
 	if config['cache_manual_pages'] and os.path.exists(config['manual_debug']):
@@ -91,18 +99,21 @@ def main(config: dict):
 		# Encode pages
 		book_content = []
 		os.makedirs(f"{config['manual_path']}/font/category", exist_ok=True)
-		simple_case = Image.open(f"{MANUAL_ASSETS_PATH}/simple_case_no_border.png")	# Load the simple case image for later use in categories pages
+		simple_case = load_simple_case_no_border(config['manual_high_resolution'])	# Load the simple case image for later use in categories pages
 		for page in manual_pages:
 			content = []
 			number = page["number"]
-			page_font = get_page_font(number)
+			raw_data: dict = page["raw_data"]
+			page_font = ""
+			if not config['manual_high_resolution']:
+				page_font = get_page_font(number)
 			name = str(page["name"])
-			raw_data = page["raw_data"]
 			titled = name.replace("_", " ").title() + "\n"
 
 			# Encode categories {'number': 2, 'name': 'Material #1', 'raw_data': ['adamantium_block', 'adamantium_fragment', ...]}
 			if page["type"] == CATEGORY:
 				file_name = name.replace(" ", "_").replace("#", "").lower()
+				page_font = get_page_font(number)
 				font_providers.append({"type":"bitmap","file":f"{config['namespace']}:font/category/{file_name}.png", "ascent": 0, "height": 130, "chars": [page_font]})
 				content.append({"text": "", "font": FONT, "color": "white"})	# Make default font for every next component
 				content.append({"text": "➤ ", "font": "minecraft:default", "color": "black"})
@@ -116,34 +127,47 @@ def main(config: dict):
 
 				# For each item in the category, get its page number and texture, then add it to the image
 				for item in raw_data:
+
+					# Get item texture
 					texture_path = f"{config['manual_path']}/items/{config['namespace']}/{item}.png"
-					item_image = Image.open(texture_path)
-					factor = 32 / item_image.size[0]
-					item_image = item_image.resize((32, int(item_image.size[1] * factor)), Image.NEAREST)
+					if os.path.exists(texture_path):
+						item_image = Image.open(texture_path)
+					else:
+						warning(f"Missing texture at '{texture_path}', using empty texture")
+						item_image = Image.new("RGBA", (1, 1), (0, 0, 0, 0))
+					if not config['manual_high_resolution']:
+						resized = careful_resize(item_image, 32)
+					else:
+						resized = Image.new("RGBA", (1, 1), (0, 0, 0, 0))	# Empty texture to use for category page
+						high_res_font = generate_high_res_font(config, item, item_image)
 
 					# Paste the simple case and the item_image
 					page_image.paste(simple_case, (x, y))
-					mask = item_image.convert("RGBA").split()[3]
-					page_image.paste(item_image, (x + 2, y + 2), mask)
-					x += 36
+					mask = resized.convert("RGBA").split()[3]
+					page_image.paste(resized, (x + 2, y + 2), mask)
+					x += simple_case.size[0]
 
 					# Add the clickEvent part to the line and add the 2 times the line if enough items
 					component = get_item_component(config, item, ["custom_model_data", "item_name", "custom_name"])
-					component["text"] = MEDIUM_NONE_FONT
+					component["text"] = MEDIUM_NONE_FONT if not config['manual_high_resolution'] else high_res_font
 					line.append(component)
 					if len(line) == config['max_items_per_row']:
-						line[-1]["text"] += "\n"
-						line[0]["text"] = SMALL_NONE_FONT * LEFT_PADDING + line[0]["text"]
-						content += line * 2
+						line.insert(0, SMALL_NONE_FONT * LEFT_PADDING)
+						content += deepcopy(line)
+						for i in range(1, len(line)):
+							line[-i]["text"] = MEDIUM_NONE_FONT
+						content += ["\n"] + line + ["\n"]
 						line = []
 						x = 2
-						y += 36
+						y += simple_case.size[1]
 				
 				# If remaining items in the line, add them
 				if len(line) > 0:
-					line[-1]["text"] += "\n"
-					line[0]["text"] = SMALL_NONE_FONT * LEFT_PADDING + line[0]["text"]
-					content += line * 2
+					line.insert(0, SMALL_NONE_FONT * LEFT_PADDING)
+					content += deepcopy(line)
+					for i in range(1, len(line)):
+						line[-i]["text"] = MEDIUM_NONE_FONT
+					content += ["\n"] + line + ["\n"]
 				
 				# Add the 2 pixels border
 				is_rectangle_shape = len(raw_data) % config['max_items_per_row'] == 0
@@ -164,8 +188,11 @@ def main(config: dict):
 				
 				# Else, generate the content for the single item in a big box
 				else:
+					if not page_font:
+						page_font = get_page_font(number)
 					generate_page_font(config, name, page_font, craft = None)
 					component = get_item_component(config, name)
+					component["text"] = NONE_FONT
 					component["text"] *= 2
 					content.append({"text": "", "font": FONT, "color": "white"})	# Make default font for every next component
 					content.append({"text": titled, "font": "minecraft:default", "color": "black", "underlined": True})
@@ -193,15 +220,25 @@ def main(config: dict):
 				for i, craft in enumerate(crafts):
 					if craft["type"] == "crafting_shapeless":
 						craft = convert_shapeless_to_shaped(craft)
-					craft_font = get_next_font()	# Unique used font for the craft
-					generate_page_font(config, name, craft_font, craft, output_name = f"{name}_{i+1}")
-					hover_text = [""]
 
-					# Append the craft font and breaklines
+					# Get breaklines
 					breaklines = 3
 					if "shape" in craft:
 						breaklines = max(2, max(len(craft["shape"]), len(craft["shape"][0])))
-					hover_text.append({"text": craft_font + "\n\n" * breaklines, "font": FONT, "color": "white"})
+
+					if not config['manual_high_resolution']:
+						craft_font = get_next_font()	# Unique used font for the craft
+						generate_page_font(config, name, craft_font, craft, output_name = f"{name}_{i+1}")
+						hover_text = [""]
+						hover_text.append({"text": craft_font + "\n\n" * breaklines, "font": FONT, "color": "white"})
+					else:
+						l = generate_craft_content(config, craft, name, "")
+						l = [l[0]] + l[2:]	# Remove craft title
+						remove_events(l)
+						replaces = {SHAPED_2X2_FONT: HOVER_SHAPED_2X2_FONT, SHAPED_3X3_FONT: HOVER_SHAPED_3X3_FONT, FURNACE_FONT: HOVER_FURNACE_FONT}
+						for k, v in replaces.items():
+							l[1] = l[1].replace(k, v)
+						hover_text = ["", l]
 
 					# Append ingredients
 					if craft.get("ingredient"):
@@ -294,41 +331,54 @@ def main(config: dict):
 		for page in manual_pages:
 			if page["type"] == CATEGORY:
 				item = page["raw_data"][0]
+
+				# Get item texture TODO
 				texture_path = f"{config['manual_path']}/items/{config['namespace']}/{item}.png"
-				item_image = Image.open(texture_path)
-				factor = 32 / item_image.size[0]
-				item_image = item_image.resize((32, int(item_image.size[1] * factor)), Image.NEAREST)
+				if os.path.exists(texture_path):
+					item_image = Image.open(texture_path)
+				else:
+					warning(f"Missing texture at '{texture_path}', using empty texture")
+					item_image = Image.new("RGBA", (1, 1), (0, 0, 0, 0))
+				if not config['manual_high_resolution']:
+					resized = careful_resize(item_image, 32)
+				else:
+					resized = Image.new("RGBA", (1, 1), (0, 0, 0, 0))	# Empty texture to use for category page
+					high_res_font = generate_high_res_font(config, item, item_image)
 
 				# Paste the simple case and the item_image
 				page_image.paste(simple_case, (x, y))
-				mask = item_image.convert("RGBA").split()[3]
-				page_image.paste(item_image, (x + 2, y + 2), mask)
-				x += 36
+				mask = resized.convert("RGBA").split()[3]
+				page_image.paste(resized, (x + 2, y + 2), mask)
+				x += simple_case.size[0]
 
 				# Add the clickEvent part to the line and add the 2 times the line if enough items
 				component = get_item_component(config, item, ["custom_model_data"])
 				component["hoverEvent"]["contents"]["components"]["item_name"] = str({"text": page["name"], "color": "white"})
 				component["clickEvent"]["value"] = str(page["number"])
-				component["text"] = MEDIUM_NONE_FONT
+				if not config['manual_high_resolution']:
+					component["text"] = MEDIUM_NONE_FONT
+				else:
+					component["text"] = high_res_font
 				line.append(component)
 				if len(line) == config['max_items_per_row']:
-					line[-1]["text"] += "\n"
-					line[0]["text"] = SMALL_NONE_FONT * LEFT_PADDING + line[0]["text"]
-					content += line * 2
+					line.insert(0, SMALL_NONE_FONT * LEFT_PADDING)
+					content += deepcopy(line) + ["\n"]
+					for i in range(1, len(line)):
+						line[-i]["text"] = MEDIUM_NONE_FONT
+					content += line + ["\n"]
 					line = []
 					x = 2
-					y += 36
+					y += simple_case.size[1]
 		
 		# If remaining items in the line, add them
 		if len(line) > 0:
-			line[-1]["text"] += "\n"
-			line[0]["text"] = SMALL_NONE_FONT * LEFT_PADDING + line[0]["text"]
-			content += line * 2
+			line.insert(0, SMALL_NONE_FONT * LEFT_PADDING)
+			content += deepcopy(line) + ["\n"]
+			for i in range(1, len(line)):
+				line[-i]["text"] = MEDIUM_NONE_FONT
+			content += line + ["\n"]
 		
 		# Add the 2 pixels border
-		BORDER_COLOR = 0xB64E2F
-		BORDER_SIZE = 2
-		BORDER_COLOR = (BORDER_COLOR >> 16) & 0xFF, (BORDER_COLOR >> 8) & 0xFF, BORDER_COLOR & 0xFF, 255
 		is_rectangle_shape = len(raw_data) % config['max_items_per_row'] == 0
 		page_image = add_border(page_image, BORDER_COLOR, BORDER_SIZE, is_rectangle_shape)
 		
@@ -349,7 +399,7 @@ def main(config: dict):
 		if not os.path.exists(icon_path):
 			error(f"Missing icon path at '{icon_path}' (needed for the manual)")
 		logo = Image.open(icon_path)
-		logo = logo.resize((256, 256), Image.NEAREST)
+		logo = careful_resize(logo, 256)
 
 		# Write the introduction text
 		content.append("\n" * 6)
@@ -367,10 +417,19 @@ def main(config: dict):
 		font_providers.append({"type":"bitmap","file":f"{config['namespace']}:font/none.png", "ascent": 8, "height": 18, "chars": [MEDIUM_NONE_FONT]})
 		font_providers.append({"type":"bitmap","file":f"{config['namespace']}:font/none.png", "ascent": 7, "height": 7, "chars": [SMALL_NONE_FONT]})
 		font_providers.append({"type":"bitmap","file":f"{config['namespace']}:font/none.png", "ascent": 0, "height": 2, "chars": [VERY_SMALL_NONE_FONT]})
+		font_providers.append({"type":"bitmap","file":f"{config['namespace']}:font/none.png", "ascent": 0, "height": 1, "chars": [MICRO_NONE_FONT]})
 		font_providers.append({"type":"bitmap","file":f"{config['namespace']}:font/none.png", "ascent": 7, "height": 16, "chars": [WIKI_NONE_FONT]})
+		font_providers.append({"type":"bitmap","file":f"{config['namespace']}:font/invisible_item.png", "ascent": 7, "height": 16, "chars": [INVISIBLE_ITEM_FONT]})
 		font_providers.append({"type":"bitmap","file":f"{config['namespace']}:font/wiki_information.png", "ascent": 8, "height": 16, "chars": [WIKI_INFO_FONT]})
 		font_providers.append({"type":"bitmap","file":f"{config['namespace']}:font/wiki_result_of_craft.png", "ascent": 8, "height": 16, "chars": [WIKI_RESULT_OF_CRAFT_FONT]})
 		font_providers.append({"type":"bitmap","file":f"{config['namespace']}:font/wiki_ingredient_of_craft.png", "ascent": 8, "height": 16, "chars": [WIKI_INGR_OF_CRAFT_FONT]})
+		if config['manual_high_resolution']:
+			font_providers.append({"type":"bitmap","file":f"{config['namespace']}:font/shaped_3x3.png", "ascent": 1, "height": 58, "chars": [SHAPED_3X3_FONT]})
+			font_providers.append({"type":"bitmap","file":f"{config['namespace']}:font/shaped_2x2.png", "ascent": 1, "height": 58, "chars": [SHAPED_2X2_FONT]})
+			font_providers.append({"type":"bitmap","file":f"{config['namespace']}:font/furnace.png", "ascent": 1, "height": 58, "chars": [FURNACE_FONT]})
+			font_providers.append({"type":"bitmap","file":f"{config['namespace']}:font/shaped_3x3.png", "ascent": -4, "height": 58, "chars": [HOVER_SHAPED_3X3_FONT]})
+			font_providers.append({"type":"bitmap","file":f"{config['namespace']}:font/shaped_2x2.png", "ascent": -2, "height": 58, "chars": [HOVER_SHAPED_2X2_FONT]})
+			font_providers.append({"type":"bitmap","file":f"{config['namespace']}:font/furnace.png", "ascent": -3, "height": 58, "chars": [HOVER_FURNACE_FONT]})
 		fonts = {"providers": font_providers}
 		with super_open(f"{config['manual_path']}/font/manual.json", "w") as f:
 			f.write(super_json_dump(fonts).replace("\\\\", "\\"))
@@ -386,7 +445,18 @@ def main(config: dict):
 	super_copy(f"{config['manual_path']}/font/category/", f"{config['build_resource_pack']}/assets/{config['namespace']}/textures/font/category/")
 	super_copy(f"{config['manual_path']}/font/page/", f"{config['build_resource_pack']}/assets/{config['namespace']}/textures/font/page/")
 	super_copy(f"{config['manual_path']}/font/wiki_icons/", f"{config['build_resource_pack']}/assets/{config['namespace']}/textures/font/wiki_icons/")
+	if config['manual_high_resolution']:
+		super_copy(f"{config['manual_path']}/font/high_res/", f"{config['build_resource_pack']}/assets/{config['namespace']}/textures/font/high_res/")
 
+	# Verify font providers and textures
+	for fp in font_providers:
+		if "file" in fp:
+			path: str = fp["file"]
+			path = path.replace(config['namespace'] + ':', f"{config['build_resource_pack']}/assets/{config['namespace']}/textures/")
+			if not os.path.exists(path):
+				error(f"Missing font provider at '{path}' for {fp})")
+			if len(fp["chars"]) < 1 or (len(fp["chars"]) == 1 and not fp["chars"][0]):
+				error(f"Font provider '{path}' has no chars")
 
 	# Finally, prepend the manual to the database
 	manual_cmd = min(x["custom_model_data"] for x in config['database'].values() if x.get("custom_model_data")) - 1		# First custom_model_data minus 1
@@ -407,7 +477,7 @@ def main(config: dict):
 
 	# Add the model to the resource pack
 	from ..resource_pack.item_models import handle_item
-	handle_item("manual", config['database']["manual"])
+	handle_item(config, "manual", config['database']["manual"])
 	vanilla_model = {"parent": "item/handheld","textures": {"layer0": "item/written_book"},"overrides": [{ "predicate": { "custom_model_data": manual_cmd}, "model": f"{config['namespace']}:item/manual" }]}
 	vanilla_model = super_json_dump(vanilla_model).replace('{"','{ "').replace('"}','" }').replace(',"', ', "')
 	write_to_file(f"{config['build_resource_pack']}/assets/minecraft/models/item/written_book.json", vanilla_model)
